#include <sstream>
#include <cmath>
#include "ezwin.h"
using namespace std ;

// This must be imported.
extern void (*fTab[])(float &, float &, float &) ;

enum color float2color(const float f)
{
	enum color colors[] = { Red, Yellow, Green, Cyan, Blue, Magenta } ;
	const int  colorn = 6 ;
	return colors[(int)((f - floor(f)) * colorn)] ;

}


void PlotPoints(SimpleWindow &W,
				void (*f)(float &, float &, float &))
{
	int i, j ;
	const int Res = 100 ;
	const float invRes = 1.0f/Res ;
	float centX = W.GetWidth()*0.5 ;
	float centY = W.GetHeight()*0.5 ;
	float maxDim = ( centX > centY ? centX : centY ) ;
	float boxDim = maxDim/Res ;
	float axLW = maxDim*0.010 ;

	// Plot the function
	for (i=-Res; i<Res; ++i)
		for(j=-Res; j<Res; ++j)
		{
			float x = i*invRes + 0.5f*invRes ;
			float y = j*invRes + 0.5f*invRes ;
			float z = 0;
			f(x, y, z) ;
			enum color c = float2color(z) ;
			Position ulP(centX+i*boxDim, centY-(j+1)*boxDim);
			Position lrP(centX+(i+1)*boxDim, centY-j*boxDim);
			W.RenderRectangle(ulP, lrP, c, false) ;
		}

	// Draw the axes
	Position CTop(centX, centY + maxDim) ;
	Position CBot(centX, centY - maxDim) ;
	Position CLef(centX - maxDim, centY ) ;
	Position CRig(centX + maxDim, centY ) ;
	W.RenderLine(CTop, CBot, Black, axLW) ;
	W.RenderLine(CLef, CRig, Black, axLW) ;

}

SimpleWindow *pW;

int funcIndex = 0 ;

Position home(0.0f, 0.0f) ;

int refreshPlot()
{
	PlotPoints(*pW, fTab[funcIndex]) ;
	return 0 ;
}

int nextPlot(const Position &Ignored)
{
	pW->Close() ;
	delete pW ;
	if (fTab[++funcIndex] == NULL)
		funcIndex = 0 ;
	ostringstream winName ;
	winName << "Function " << funcIndex+1 ;
	pW = new SimpleWindow(winName.str(), 18.0f, 18.0f, home) ;
	pW->Open() ;
	pW->SetMouseClickCallback(nextPlot) ;
	pW->SetRefreshCallback(refreshPlot) ;
	PlotPoints(*pW, fTab[funcIndex]) ;
	return 0 ;
}

int ApiMain() 
{
	pW = new SimpleWindow("Function 1", 18.0f, 18.0f, home) ;
	pW->Open() ;
	pW->SetMouseClickCallback(nextPlot) ;
	pW->SetRefreshCallback(refreshPlot) ;
	PlotPoints(*pW, fTab[0]) ;
	return 0 ;
}

int ApiEnd()
{
	pW->Close() ;
	return 0 ;
}
